package {
	
	import flash.display.Bitmap;
	import flash.display.Sprite;
	
	import Box2D.Collision.Shapes.b2CircleShape;
	import Box2D.Collision.Shapes.b2PolygonShape;
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2BodyDef;
	import Box2D.Dynamics.b2DebugDraw;
	import Box2D.Dynamics.b2FixtureDef;
	import Box2D.Dynamics.b2World;
	
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.text.TextField;
	import starling.textures.Texture;
	
	
	/**
	 * @author Joshua Granick
	 */
	public class TestBox2D extends starling.display.Sprite {
		
		
		private static var PHYSICS_SCALE:Number = 1 / 30;
		
		[Embed(source="/../embeds/basketball.png")]
		private const BallRef:Class;
		
		private var _ball:Bitmap;
		private var _ballTexture:Texture;
		
		private var PhysicsDebug:flash.display.Sprite;
		private var World:b2World;
		private var _count:TextField;
		private var _ballCount:int;
		
		private const _debug:Boolean = false;
		
		public function TestBox2D () {
			
			super ();
			
			var background:Quad = new Quad(320, 480, 0xDCDCDC);
			addChild(background);
			
			_count = new TextField(120, 24, 'Box2D - ball : 0');
			_count.x = 190;
			addChild(_count);
			
			_ball = new BallRef;
			_ballTexture = Texture.fromBitmap(_ball);
			
			initialize ();
			
		}
		
		private function initialize ():void {
			
			World = new b2World (new b2Vec2 (0, 20.0), true);
			
			if(_debug){
				PhysicsDebug = new flash.display.Sprite ();
				Starling.current.nativeStage.addChild (PhysicsDebug);
				PhysicsDebug.alpha = 0.5;
				
				var debugDraw:b2DebugDraw = new b2DebugDraw ();
				debugDraw.SetSprite (PhysicsDebug);
				debugDraw.SetDrawScale (1 / PHYSICS_SCALE);
				debugDraw.SetFlags (b2DebugDraw.e_shapeBit);
				
				World.SetDebugDraw (debugDraw);
			}
			
			createBox (0, 470, 800, 20, false);
			createBox (-10, 250, 20, 500, false);
			createBox (330, 250, 20, 500, false);
			
			addEventListener (Event.ENTER_FRAME, this_onEnterFrame);
			addEventListener(TouchEvent.TOUCH, onTouch);
		}
		
		private function onTouch(event:TouchEvent):void
		{
			var touchesBegan:Vector.<Touch> = event.getTouches(this,TouchPhase.BEGAN);
			var touchesMove:Vector.<Touch> = event.getTouches(this,TouchPhase.MOVED);
			if(touchesMove.length>0){
				addBall(touchesMove[0].globalX, touchesMove[0].globalY);
			}
			
			if(touchesBegan.length>0){
				//if(isFlattened==false) flatten();
				//else unflatten();
			}
			
		}
		
		private function createBox (x:Number, y:Number, width:Number, height:Number, dynamicBody:Boolean):void {
			
			var bodyDefinition:b2BodyDef = new b2BodyDef ();
			bodyDefinition.position.Set (x * PHYSICS_SCALE, y * PHYSICS_SCALE);
			
			if (dynamicBody) {
				
				bodyDefinition.type = b2Body.b2_dynamicBody;
				
			}
			
			var polygon:b2PolygonShape = new b2PolygonShape ();
			polygon.SetAsBox ((width / 2) * PHYSICS_SCALE, (height / 2) * PHYSICS_SCALE);
			
			var fixtureDefinition:b2FixtureDef = new b2FixtureDef ();
			fixtureDefinition.shape = polygon;
			
			var body:b2Body = World.CreateBody (bodyDefinition);
			body.CreateFixture (fixtureDefinition);
			
		}
		
		
		private function createCircle (x:Number, y:Number, radius:Number, dynamicBody:Boolean):void {
			
			var bodyDefinition:b2BodyDef = new b2BodyDef ();
			bodyDefinition.position.Set (x * PHYSICS_SCALE, y * PHYSICS_SCALE);
			if (dynamicBody) {
				
				bodyDefinition.type = b2Body.b2_dynamicBody;
				
			}
			
			var circle:b2CircleShape = new b2CircleShape (radius * PHYSICS_SCALE);
			
			var fixtureDefinition:b2FixtureDef = new b2FixtureDef ();
			fixtureDefinition.shape = circle;
			fixtureDefinition.density = .5;
			fixtureDefinition.friction = .2;
			fixtureDefinition.restitution = .5;
			
			var ball:Image = new Image(_ballTexture);
			ball.pivotX = ball.width/2;
			ball.pivotY = ball.height/2;
			addChild(ball);
				
			var body:b2Body = World.CreateBody (bodyDefinition);
			body.CreateFixture (fixtureDefinition);
			body.SetUserData( ball );
		}
		
		private function addBall(x:Number, y:Number):void
		{
			createCircle(x, y, 12, true);
			
			_count.text = 'Box2d ball : '+(++_ballCount);
		}
		
		private function this_onEnterFrame (event:Event):void {
			World.Step (1 / 60, 10, 10);
			World.ClearForces ();
			if(_debug){
				World.DrawDebugData ();
			}
			
			if(Math.random() < 0.03){
				addBall(Math.random() * 300, 20);
			}
			
			for(var b:b2Body = World.GetBodyList(); b; b=b.GetNext()) {  
				if (b.GetUserData() != null) {
					var image:Image = b.GetUserData() as Image;
					image.x = b.GetPosition().x / PHYSICS_SCALE;
					image.y = b.GetPosition().y / PHYSICS_SCALE;
					image.rotation = b.GetAngle();
				}        
			}
		}
		
		
	}
}


