package
{

	
	import flash.display.Bitmap;
	
	import nape.geom.Vec2;
	import nape.phys.Body;
	import nape.phys.BodyList;
	import nape.phys.BodyType;
	import nape.phys.Material;
	import nape.shape.Circle;
	import nape.shape.Polygon;
	import nape.space.Space;
	import nape.util.ShapeDebug;
	
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.text.TextField;
	import starling.textures.Texture;
	
	public class TestNape extends Sprite
	{
		[Embed(source="/../embeds/basketball.png")]
		private const BallRef:Class;
		
		private var _ball:Bitmap;
		private var _ballTexture:Texture;
		
		private var _box:Quad;
		private var _world:Space;
		private var debug:ShapeDebug;
		private var _count:TextField;
		private var _ballCount:int = 0;
		
		private const _debug:Boolean = false;
		
		public function TestNape()
		{
			super();
			
			var background:Quad = new Quad(320, 480, 0xDCDCDC);
			addChild(background);
			
			_count = new TextField(120, 24, 'Nape - ball : 0');
			_count.x = 190;
			addChild(_count);
			
			
			_ball = new BallRef;
			_ballTexture = Texture.fromBitmap(_ball);
			
			_world = new Space(new Vec2(0, 1000));
			
			var floor:Body = new Body(BodyType.STATIC);
			floor.shapes.add(new Polygon(Polygon.rect(0, 460, 800, 20)));
			floor.shapes.add(new Polygon(Polygon.rect(-20, 0, 20, 500)));
			floor.shapes.add(new Polygon(Polygon.rect(320, 0, 20, 500)));
			floor.space = _world;
			
			if(_debug){
				debug = new ShapeDebug(320, 480);
				debug.thickness =  2;
				Starling.current.nativeOverlay.addChild(debug.display);
			}
			
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			addEventListener(TouchEvent.TOUCH, onTouch);
		}
		
		private function onTouch(event:TouchEvent):void
		{
			var touchesBegan:Vector.<Touch> = event.getTouches(this,TouchPhase.BEGAN);
			var touchesMove:Vector.<Touch> = event.getTouches(this,TouchPhase.MOVED);
			if(touchesMove.length>0){
				addBall(touchesMove[0].globalX, touchesMove[0].globalY);
			}
		}		
		
		private function onEnterFrame(e:Event):void {
			if(_debug){
				debug.clear();
				_world.step(1/60);
				debug.draw(_world);	
			} else {
				_world.step(1/60);
			}
			
			if(Math.random() < 0.03){
				addBall(Math.random() * 300, -10);
			}
			// looping though bodies
			var bodies:BodyList=_world.bodies;
			for (var i:int = 0; i < bodies.length; i++) {
				var body:Body=bodies.at(i);
				if(body.userData.image!=null){
					// adjusting graphic asset position
					body.userData.image.x=body.position.x;
					body.userData.image.y=body.position.y;
					body.userData.image.rotation=body.rotation;
				}
			}
		}
		
		private function addBall(x:Number, y:Number):void
		{
			var ball:Body = new Body(BodyType.DYNAMIC, new Vec2(x, y));
			ball.shapes.add(new Circle(_ball.width/2, null, new Material(.5, 2, 2, .5)));
			ball.space = _world;
			ball.userData.image = new Image(_ballTexture);
			ball.userData.image.pivotX = ball.userData.image.width/2;
			ball.userData.image.pivotY = ball.userData.image.height/2;
			addChild(ball.userData.image);
			
			_count.text = 'Nape - ball : '+(++_ballCount);
		}
	}
}